/*
 * Project.......: WTINOO
 * File..........: FUNTINIO.H
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface of some IO classes used by TIN objects.
 */

#ifndef FUNTINIO_H
#define FUNTINIO_H

#include "tinio.h"

class KeyBuffer
{
private:
	char c;
	int read;
	int linked;
public:
	KeyBuffer(int nlinked);
	bool isNonEmpty();
	char getKey();
};

class KeyBufferLinked
{
protected:
	KeyBuffer *keybuffer;
public:
	KeyBufferLinked(KeyBuffer *newkeybuffer=NULL) { setKeyBuffer(newkeybuffer); }
	void setKeyBuffer(KeyBuffer *newkeybuffer) { keybuffer=newkeybuffer; }
};

class KeyInput: public Input , public KeyBufferLinked
{
private:
	char onkey,offkey;
public:
    KeyInput(const char *newname, char newonkey, char newoffkey, KeyBuffer *buffer);
    io_val eval();
};

class UpKeyInput: public Input , public KeyBufferLinked
{
private:
	char upkey;
public:
    UpKeyInput(const char *newname, char newupkey, KeyBuffer *buffer);
    io_val eval();
};

class AlertSoundOutput: public ThreadOutput
{
public:
    AlertSoundOutput(const char *newname);
    void action(bool value);
};

class BoundedMelodyOutput: public BoundedThreadOutput
{
public:
    BoundedMelodyOutput(const char *newname, unsigned int timeout);
    void action(bool value);
};

class ScreenOutput: public Output
{
private:
	bool previous;
public:
    ScreenOutput(const char *newname);
    void action(bool value);
};

class BeepOutput: public Output
{
public:
    BeepOutput(const char *newname);
    void action(bool value);
};

#endif //FUNTINIO_H