/*
 * Project.......: WTINOO
 * File..........: RTEXEC.H
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface for the RTexec class. This class performs the real-time loop
 *		that uses TIN objects.
 */

#ifndef RTEXEC_H
#define RTEXEC_H

#include "tinoo.h"
#include "tinio.h"
#include "thrsem.h"

/*
	RTexec Class: Main class that includes the containers for the TIN instances
	and for the IODevice. It creates and makes run a thread that work as the 
	main sequencer for the events.
*/

class RTexec : private Thread
{
private:
	unsigned long delta; //Is the resolution of our engine and is in milliseconds

	TINBox tinbox;	//Container of the TIN objects
	InBox inbox;	//Container of the input devices
	OutBox outbox;	//Container of the output devices

	/* run: thread specification (main time loop sequence) */
	int run();

public:
	/* Constructor: it fixes the resolution, and the max size of each containers */
	RTexec(unsigned long rtdelta, int num_tin, int num_in, int num_out);

	/* addtin: add a TIN object */
	void addTIN(TIN *in_tin);

	/*linkIODev: this function has to prototypes for different kinds of call.
	  it provides to add the IODev in the right (In or Out) container
	  and to link the device to the interested TIN object. */
	void linkIODev(IODev *iodev, const char *tin_name, const char *io_name);
	void linkIODev(IODev *iodev, TIN *tin, const char *io_name);

	void addIODev(IODev *iodev);

	/* set/getdelta: it is possible to set and read the resolution */
	void setDelta(unsigned long delta);
	unsigned long getDelta();
	
	/* setReadyToStart: this function initializes everything before giving 
	   the start command it creates the threads for each TIN object and reset 
	   the time. */
	void setReadyToStart();

	/* startExecution: this function make run the main cycle that will perform 
	   the syncronization between TIN objects and IO devices. */
	void startExecution();

	void sequentialExecution(FILE *logfile_fp=NULL);
};

#endif	//RTEXEC_H