/*
 * Project.......: WTINOO
 * File..........: THRSEM.H
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface for the Thread class. This class is included to make portable
 *		on different operative system (Windows NT, Linux,...).
 */

#ifndef THRSEM_H
#define THRSEM_H

class Thread
{
private:
	void *thread_h;	 //Thread handle: it allows to control the thread (wait, start, kill)
	int thread_prio; //Contains the thread priority (Windows for now)
	unsigned long thread_id;		//Id returned by the Windows API create function
	unsigned long suspend_count;	//Suspend count to know if the thread is running
	bool created;

	/* run: method designed to be overrided in every custom thread */
	virtual int run();

	/* toExec: static fuction needed for give to the create thread function a
	   fisical pointer. It just call the run method of thread_ist. */
	static void toExec(Thread *thread_ist);

public:
	/* Constructor: it does not create the fisical thread. */
	Thread();
	
	/* create: creates the thread suspended */
	void create();
	
	/* start: starts the thread after a creation or after a wait, as the other 
	   following methods it works only if the thread is created. */
	void start();

	/* setPriority: set the priority (Windows defines for now)  */
	void setPriority(int prio);

	/* wait: makes the thread wait */
	void wait();

	/* kill: terminates the thread */
	void kill();

	/* sleep: it makes the thread sleep for a while */
	void sleep(unsigned long millis);

	/* isRunning: returns if the susspend count is 0 */
	bool Running();

	/* isRunning: returns if the susspend count is 0 */
	bool isCreated();

	/* sleep: it makes the thread sleep for a while */
	unsigned long getTickCount();
};

class Mutex
{
private:
	void *mutex_h;	 //Mutex handle: it allows to perform the mutual exclusion;
public:
	Mutex();		//Constructor..
void take();	//Grab the lock if it is free
	void give();	//Release the lock
};

class Semaphore
{
private:
	void *semaphore_h;	 //Semaphore handle;
	Mutex state_lock;
protected:
	long semcount;	//Semaphore state.
	bool iswaiting;	//Some Threads are waiting...
public:
	Semaphore();	//Constructor..
	void wait();	//wait for the green light
	void signal();	//green light
	long getSemCount();
	bool Waiting();
};

#endif // THRSEM_H
