/*
 * Project.......: WTINOO
 * File..........: TINEXC.H
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface for the exceptions.
 */

#ifndef TINEXC_H
#define TINEXC_H

#include <stdlib.h>
#include <string.h>

class Exception
{
public:
	virtual const char *what() { return(NULL); }
};

class TimeOutException : public Exception
{
	char tinname[25];
public:
	TimeOutException(const char *name) { strcpy(tinname,name); }
	const char *getTINName() { return(tinname); }
	virtual const char *what() { return("Time-out"); }
};

class InputErrorException : public Exception
{
	char inputname[25];
public:
	InputErrorException(const char *name) { strcpy(inputname,name); }
	const char *getInputName() { return(inputname); }
	virtual const char *what() { return("Input Error"); }
};

class ThreadException : public Exception
{
public:
	virtual const char *what() { return("Thread Error"); }
};

class LinkErrorException : public Exception
{
public:
	virtual const char *what() { return("Link Error"); }
};

class MallocErrorException : public Exception
{
public:
	virtual const char *what() { return("Memory Allocation Error"); }
};

class FileErrorException : public Exception
{
	char filename[15];
public:
	FileErrorException(const char *err_filename) { strcpy(filename,err_filename); }
	const char *getFileName() { return(filename); }
	virtual const char *what() { return("File Error"); }
};

class ParserErrorException : public Exception
{
	char message[80];
public:
	ParserErrorException(const char *err_message) { strcpy(message,err_message); }
	const char *getMessage() { return(message); }
	virtual const char *what() { return("Parser Error"); }
};

#endif //TINEXC_H