/*
 * Project.......: WTINOO
 * File..........: TINTHRSEM.H
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface of Threads & Semaphores controllable by TIN objects.
 */

#ifndef TINTHRSEM_H
#define TINTHRSEM_H

#include "thrsem.h"
#include "tinio.h"

#define MAX_PARAMS 100

class TINThread;
class TINSemaphore;

class TINThreadChildObj : public ChildObj
{
public:
	void setParent(TINThread *parentthread) { ChildObj::setParent(parentthread); }
	TINThread *getParent() { return((TINThread *)ChildObj::getParent()); }
};

class TINSemaphoreChildObj : public ChildObj
{
public:
	void setParent(TINSemaphore *parentsemaphore) { ChildObj::setParent(parentsemaphore); }
	TINSemaphore *getParent() { return((TINSemaphore *)ChildObj::getParent()); }
};

class StartOutput : public Output, public TINThreadChildObj
{
public:
	StartOutput(const char *newname, const char *suffix);
	void action(bool val);
};

class KillOutput : public Output, public TINThreadChildObj
{
public:
	KillOutput(const char *newname, const char *suffix);
	void action(bool val);
};

class RunningInput : public Input, public TINThreadChildObj
{
public:
	RunningInput(const char *newname, const char *suffix);
	io_val eval();
};

class TINThread : public NamedObj , public Thread
{
protected:
	StartOutput startreq;
	KillOutput killreq;
	RunningInput runningstate;
	CommandInput succeeded;

	unsigned long minduration;

	int run();

	virtual bool toRun() { return(true); }

public:
	TINThread(const char *newname, unsigned long minduration=0L);
	void kill();
	Output *getStart() { return ((Output *)&startreq); }
	Output *getKill() { return ((Output *)&killreq); }
	Input *getRunning() { return ((Input *)&runningstate); }
	Input *getSucceeded() { return ((Input *)&succeeded); }
};

class TINThreadWithSemaphores : public TINThread
{
private:
	TINSemaphore **sem_array;
	int nsem;
protected:
	void waitOnSem(int pos);
public:
	TINThreadWithSemaphores(const char *newname,int maxsem, 
		unsigned long newminduration=0L);
	void addSemaphore(TINSemaphore *sem, int pos);
	TINSemaphore *getSemaphore(const char *sem_name);
	Input *getInput(char *name, int nparams, char paramlist[][100]);
	Output *getOutput(char *name, int nparams, char paramlist[][100]);
};

class SignalOutput : public Output, public TINSemaphoreChildObj
{
public:
	SignalOutput(const char *newname, const char *suffix);
	void action(bool val);
};

class WaitingInput : public Input, public TINSemaphoreChildObj
{
public:
	WaitingInput(const char *newname, const char *suffix);
	io_val eval();
};

class TINSemaphore : public Semaphore , public NamedObj
{
private:
	SignalOutput signalreq;
	WaitingInput waitingstate;
public:
	TINSemaphore(const char *newname);
	Output *getSignal() { return &signalreq; }
	Input *getWaiting() { return &waitingstate; }
};

#endif //THRSEM_H
