/*
  TINX - Temporal Inference Network eXecutor
  Design & coding by Andrea Giotti, 1998-1999
*/

/*
  Real-time version update: Design & coding by Davide Rogai, 2001
*/

//All the huge comments was placed by Davide Rogai for the C++ embedding

#ifndef TINX_H
#define TINX_H

#include "tinio.h"
#include "thrsem.h"

//#define NDEBUG

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <assert.h>


#ifndef __cplusplus
typedef char bool;
#endif

#ifndef TRUE
#define FALSE 0
#define TRUE (!FALSE)
#endif

typedef int d_time;

#define TIME_FMT "%d"

typedef enum link_code
{
  no_link = -1,
  parent,
  left_son,
  right_son,
  LINK_CODES_NUMBER
} link_code;

typedef struct node node;

typedef struct arc
{
  node *vp;
  link_code lc;
} arc;

typedef struct event
{
  arc e;
  unsigned int t;
} event;

#define ev_eq(R, S) ((R).e.vp == (S).e.vp && (R).e.lc == (S).e.lc && \
                     (R).t == (S).t)

#if defined NDEBUG
#define valid(S) ((S).e.vp)
#else
#define valid(S) ((S).e.vp && (S).e.lc >= 0 && (S).e.lc < LINK_CODES_NUMBER && \
                  (S).t >= 0 && (S).t < kb->base_time->getSize())
#endif

typedef struct record
{
  bool stated;
  bool chosen;
  event other;
  event next;
  event other_todel;
  event next_todel;
} record;

typedef enum stream_class
{
  input_stream,
  output_stream,
  STREAM_CLASSES_NUMBER
} stream_class;

typedef struct stream stream;

typedef struct link
{
  arc e;
  /* Attributes of arc_neg(e) */
  record *history;
  stream *io_stream[STREAM_CLASSES_NUMBER];
} link;

#define MAX_STRLEN 32
#define MAX_NAMELEN_C "23"

struct stream
{
  char name[MAX_NAMELEN];
  stream_class sclass;
  link *pin;
  d_time offset;
    
  IODev *io;
  
  stream *next_ios;
  stream *prev_ios;
  bool open;
};

typedef enum node_class
{
  null_op = -1,
  gate,
  joint,
  delay,
  NODE_CLASSES_NUMBER
} node_class;

#define CLASS_SYMBOLS "GJD"

struct node
{
  char name[MAX_NAMELEN];
  node_class nclass;
  d_time k;
  link pin[LINK_CODES_NUMBER];
  node *vp;
};

typedef struct hash_list hash_list;

struct hash_list
{
	node *vp;
	hash_list *next;
};

#define link_of(E) ((E).vp->pin[(E).lc])
#define arc_neg(E) (link_of(E).e)
#define record_of(S) (link_of((S).e).history[(S).t])
#define stated(S) (record_of(S).stated)
#define chosen(S) (record_of(S).chosen)
#define other(S) (record_of(S).other)
#define next(S) (record_of(S).next)
#define other_todel(S) (record_of(S).other_todel)
#define next_todel(S) (record_of(S).next_todel)

#define HASH_SIZE 8191 /* Prime */
#define HASH_FMT "%X"
#define hash(X) (strtoul((X), NULL, 36) % HASH_SIZE)

typedef struct k_base
{
  event focus;
  node *network;
  hash_list *table[HASH_SIZE];
  stream *io_stream[STREAM_CLASSES_NUMBER];
  bool strictly_causal;
  bool soundness_check;
  bool trace_focus;
  bool echo_stdout;
  FILE *logfp;
  bool idle;

  VirtualRoundArray *base_time;
  Mutex *base_lock;
  Mutex *time_lock;
  bool output_ready;

  event todel;
  unsigned int input_time;
  unsigned int output_time;
  unsigned int infer_horiz;
  unsigned int max_delay;
} k_base;

#define BLANKS " \t\n\r"
#define SEPARATORS BLANKS"(,;:?!)@"
#define NAME_FMT "%"MAX_NAMELEN_C"[^"SEPARATORS"]"
#define FUN_FMT "%s"
#define OP_FMT "%c"
#define ARG_FMT TIME_FMT

#define DEFAULT_NAME "default"
#define STATE_SUFFIX "_ic"
#define LOG_SUFFIX "_log"
#define NETWORK_EXT ".tin"
#define EVENT_LIST_EXT ".evl"
#define STREAM_EXT ".io"

#define LO_CHAR '0'
#define HI_CHAR '1'
#define END_CHAR '.'
#define TERM_CHAR '\x1b' /* Escape */

/* Protos */

event ev_neg(event s);
arc arc_between(node *vp, node *wp);

void state(k_base *kb, event s);
event choose(k_base *kb);
void process(k_base *kb, event s);
void move_base_time(k_base *kb);
bool infloop(k_base *kb);

void trace(k_base *kb);

stream *open_stream(char *name, stream_class sclass, arc e, d_time offset);
void close_stream(stream *ios)
;
bool input_events(k_base *kb);
bool output_events(k_base *kb);

node *alloc_node(char *name);
node *name2node(k_base *kb, char *name);

void thread_network(node *network);
k_base *open_base(const char *base_name, unsigned int tw_size,char *logfile_name,
                  bool strictly_causal, bool soundness_check, bool echo_stdout);
void close_base(k_base *kb);
void init_state(k_base *kb, char *state_name);

void trap(void);

/* End of protos */

#endif //TINX_H
