/*
 * Project.......: WTINOO
 * File..........: XTINIO.HPP
 * Author........: Rogai Davide
 * Last review date: 24/05/2001
 * Description:
 *		Interface for some IO custom classes derived from the base IO devices.
 */

#ifndef XTINIO_H
#define XTINIO_H

#include "tinio.h"
#include <stdio.h>

#define MAX_FILENAMELEN 8
#define IOFILE_EXT ".io"

#define LO_CHAR '0'
#define HI_CHAR '1'
#define NO_CHAR ' '
#define ERR_CHAR 'E'
#define END_CHAR '.'

/*
   Example of an extention of the Input/Output classes
   the communication is perform with text files
*/

class FileInput : public Input
{
private:
	char filename[MAX_FILENAMELEN+3+1];
	FILE *fp;

public:
	FileInput(const char *name);
	io_val eval();
};

class FileOutput : public Output
{
private:
	char filename[MAX_FILENAMELEN+3+1];
	FILE *fp;

public:
	FileOutput(const char *name);
	void action(bool value);
	void onNone();
	void onError();
	virtual void close();
};

class FeedbackFileOutput : public FileOutput
{
private:
	CommandInput feedback;
public:
	FeedbackFileOutput(const char *newname)
		: FileOutput(newname), feedback(newname,"feedback") {}
	Input *getFeedback() { return(&feedback); }
	void change();
	void close();
};

#endif //XTINIO_H