/*
 * Project.......: CROSSROAD CONTROL SIMULATOR
 * File..........: cross_contr_custom.h
 * Author........: Rogai Davide
 * Last review date: 16/05/2002
 * Description:
 *		Base generated by TILCONF... edited to link the controller to
 *		the traffic light environment
 */

#ifndef CROSS_CONTR_CUSTOM_H
#define CROSS_CONTR_CUSTOM_H

#include "tinoo.h"
#include "tinio.h"
#include "tinthrsem.h"

#include "traffic_req.h"
#include "traffic_light.h"
#include "traffic_panel.h"

#define SAMPLE_TIME 400

#define MAX_TIN 1
#define MAX_IN 11
#define MAX_OUT 14

#define MAX_SEMS 0

//Custom input class to handle with TrafficReq istance (button listeners)
class ReqInput : public Input
{
private:
	TrafficReq *req;
public:
	ReqInput(const char *name) : Input(name) { req=NULL;}
	io_val eval();
	void linkReq(TrafficReq *link) { req=link; }
};

class Input_disable_request : public ReqInput
{
public:
    Input_disable_request() : ReqInput("disable_request") {}
};

class Input_emergency_request_1 : public ReqInput
{
public:
    Input_emergency_request_1() : ReqInput("emergency_request_1") {}
};

class Input_emergency_request_2 : public ReqInput
{
public:
    Input_emergency_request_2() : ReqInput("emergency_request_2") {}
};

//Custom output class to control 2 light (of the same color)
class LightOutput : public Output
{
private:
	Light *light1;
	Light *light2;
public:
	LightOutput(const char *name) : Output(name) { light1=light2=NULL;}
	void action(bool val);
	void linkLight1(Light *link) { light1=link; }
	void linkLight2(Light *link) { light2=link; }
};

class Output_green_signal_1 : public LightOutput
{
public:
    Output_green_signal_1() : LightOutput("green_signal_1") {}
};

class Output_yellow_signal_1 : public LightOutput
{
public:
    Output_yellow_signal_1() : LightOutput("yellow_signal_1") {}
};

class Output_red_signal_1 : public LightOutput
{
public:
    Output_red_signal_1() : LightOutput("red_signal_1") {}
};

class Output_green_signal_2 : public LightOutput
{
public:
    Output_green_signal_2() : LightOutput("green_signal_2") {}
};

class Output_yellow_signal_2 : public LightOutput
{
public:
    Output_yellow_signal_2() : LightOutput("yellow_signal_2") {}
};

class Output_red_signal_2 : public LightOutput
{
public:
    Output_red_signal_2() : LightOutput("red_signal_2") {}
};

//Custom thread class to control 2 traffic panel
class PanelThread : public TINThreadWithSemaphores
{
private:
	TrafficPanel *panel1;
	TrafficPanel *panel2;
	char scrolltext[100];
	char scrolled[200];

	bool toRun();
public:
	PanelThread(const char *name) : TINThreadWithSemaphores(name,MAX_SEMS,SAMPLE_TIME) {}
	void linkPanel1(TrafficPanel *link) { panel1=link; }
	void linkPanel2(TrafficPanel *link) { panel2=link; }
	void setScrollText(const char *text);
};

class Thread_info_1 : public PanelThread
{
public:
    Thread_info_1() : PanelThread("info_1") { setScrollText("traffic info, jam, crushes...  "); }
};

class Thread_info_2 : public PanelThread
{
public:
    Thread_info_2() : PanelThread("info_2") { setScrollText("info info info info...  "); }
};

class Thread_commerce_1 : public PanelThread
{
public:
    Thread_commerce_1() : PanelThread("commerce_1") { setScrollText("commercial spot, buy that, buy everything...  "); }
};

class Thread_commerce_2 : public PanelThread
{
public:
    Thread_commerce_2() : PanelThread("commerce_2") { setScrollText("spot spot spot spot...  "); }
};

#endif
