/*
 * Project.......: CROSSROAD CONTROL SIMULATOR
 * File..........: cross_contr_sim.cpp
 * Author........: Rogai Davide
 * Last review date: 16/05/2002
 * Description:
 *		Main source for a WIN32 application that simulate a crossoroad controller
 */

#include <windows.h>
#include "resource.h"

// Environment class headers
#include "traffic_light.h"
#include "traffic_req.h"
#include "traffic_panel.h"

// Control class header
#include "cross_contr.h"


// Getting global IO device declared in the automatic generated code by TILCONF

// GETTING INPUT
extern Input_disable_request disable_request;
extern Input_emergency_request_1 emergency_request_1;
extern Input_emergency_request_2 emergency_request_2;

// GETTING OUTPUT
extern Output_green_signal_1 green_signal_1;
extern Output_green_signal_2 green_signal_2;
extern Output_yellow_signal_1 yellow_signal_1;
extern Output_yellow_signal_2 yellow_signal_2;
extern Output_red_signal_1 red_signal_1;
extern Output_red_signal_2 red_signal_2;

// GETTING THREAD
extern Thread_info_1 info_1;
extern Thread_info_2 info_2;
extern Thread_commerce_1 commerce_1;
extern Thread_commerce_2 commerce_2;

#define MAX_LOADSTRING 100


// BUTTON ID
#define ID_DISABLE_BTN 1000
#define ID_EMERGENCY1_BTN 1001
#define ID_EMERGENCY2_BTN 1002


// Global Variables:
HINSTANCE hInst;									// current instance
HWND hWnd;											// main window
HWND hDisableBtn, hEmergency1Btn, hEmergency2Btn;	//buttons
HBITMAP hCrossroadBmp;								//crossroadbitmap resource
TCHAR szTitle[MAX_LOADSTRING];
TCHAR szWindowClass[MAX_LOADSTRING];

// Global instances of the traffic light with their positions
TrafficLight Road1F(315,265,0),
			Road1B(177,68,1),
			Road2F(330,114,2),
			Road2B(133,248,3);

// Global instances of the request with their messages
TrafficReq disableReq("Disable","Enable"),
			emergency1Req("Call Emergency","Back Normal"),
			emergency2Req("Call Emergency","Back Normal");

// Global instances of panels with their positions
TrafficPanel Panel1F(338,294,0),
			Panel1B(73,68,1),
			Panel2F(358,10,2),
			Panel2B(133,270,3);


// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_TRAFFIC_LIGHT, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_TRAFFIC_LIGHT);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}


ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_TRAFFIC_LIGHT);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_TRAFFIC_LIGHT;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	// Store instance handle in our global variable
	hInst = hInstance;

	// Creating Windows... MAIN & BUTTONS

	hWnd = CreateWindow(szWindowClass, szTitle,WS_SYSMENU,
      CW_USEDEFAULT, CW_USEDEFAULT, 520, 430, NULL, NULL, hInstance, NULL);
	hDisableBtn = CreateWindow( "BUTTON", disableReq.getMsg(), WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON, 
		10, 10, 100, 25, hWnd, NULL, (HINSTANCE) GetWindowLong(hWnd, GWL_HINSTANCE), NULL);
	hEmergency1Btn = CreateWindow( "BUTTON", emergency1Req.getMsg(), WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON, 
		315, 358, 120, 25, hWnd, NULL, (HINSTANCE) GetWindowLong(hWnd, GWL_HINSTANCE), NULL);
	hEmergency2Btn = CreateWindow( "BUTTON", emergency2Req.getMsg(), WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON, 
		385, 110, 120, 25, hWnd, NULL, (HINSTANCE) GetWindowLong(hWnd, GWL_HINSTANCE), NULL); 

	// Assigning ID to buttons for CALLBACK handling
	SetWindowLong(hDisableBtn,GWL_ID,ID_DISABLE_BTN);
	SetWindowLong(hEmergency1Btn,GWL_ID,ID_EMERGENCY1_BTN);
	SetWindowLong(hEmergency2Btn,GWL_ID,ID_EMERGENCY2_BTN);
	
	//Loading Background bitmap
	hCrossroadBmp=LoadBitmap(hInst,MAKEINTRESOURCE(CROSSROAD));

	if (!hWnd)
	{
		return FALSE;
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	//Calling config() function
	//...this provides to link all the IO instances to the logic kernel
	config();

	//Linking the real environment to the Io devices (input, output and thread)

	// INPUT LINKING
	disable_request.linkReq(&disableReq);
	emergency_request_1.linkReq(&emergency1Req);
	emergency_request_2.linkReq(&emergency2Req);

	// OUTPUT LINKING
	green_signal_1.linkLight1(Road1F.linkGreen());
	green_signal_1.linkLight2(Road1B.linkGreen());

	green_signal_2.linkLight1(Road2F.linkGreen());
	green_signal_2.linkLight2(Road2B.linkGreen());

	yellow_signal_1.linkLight1(Road1F.linkYellow());
	yellow_signal_1.linkLight2(Road1B.linkYellow());

	yellow_signal_2.linkLight1(Road2F.linkYellow());
	yellow_signal_2.linkLight2(Road2B.linkYellow());

	red_signal_1.linkLight1(Road1F.linkRed());
	red_signal_1.linkLight2(Road1B.linkRed());

	red_signal_2.linkLight1(Road2F.linkRed());
	red_signal_2.linkLight2(Road2B.linkRed());

	// THREAD LINKING
	info_1.linkPanel1(&Panel1F);
	info_1.linkPanel2(&Panel1B);

	commerce_1.linkPanel1(&Panel1F);
	commerce_1.linkPanel2(&Panel1B);
   
	info_2.linkPanel1(&Panel2F);
	info_2.linkPanel2(&Panel2B);

	commerce_2.linkPanel1(&Panel2F);
	commerce_2.linkPanel2(&Panel2B);


	//Running the controller
	executor.setReadyToStart();
	executor.startExecution();

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	RECT rt;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmEvent)
			{
				// A push button was clicked...
				case BN_CLICKED:
					switch (wmId)
					{
					// toggling the request
					// ...to change input signal
					case ID_DISABLE_BTN:
						disableReq.toggle();
						SetWindowText(hDisableBtn,disableReq.getMsg());
						break;
					case ID_EMERGENCY1_BTN:
						emergency1Req.toggle();
						SetWindowText(hEmergency1Btn,emergency1Req.getMsg());
						break;
					case ID_EMERGENCY2_BTN:
						emergency2Req.toggle();
						SetWindowText(hEmergency2Btn,emergency2Req.getMsg());
						break;
					}
			}

				// A menu item was clicked...
			switch (wmId)
			{
				case IDM_ABOUT:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;
				
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
			// Window repainting...
			case WM_PAINT:
			{
			hdc = BeginPaint(hWnd, &ps);

			GetClientRect(hWnd, &rt);
			
			// Painting crossroad background
			HDC hdcCrossRoad=CreateCompatibleDC(hdc);
			SelectObject(hdcCrossRoad, hCrossroadBmp);
			BitBlt(hdc,0,0,rt.right-rt.left,rt.bottom-rt.top,hdcCrossRoad,
				           0,0,SRCCOPY);
			DeleteObject(hdcCrossRoad);

			// Drawing traffic light
			setTrafficHDC(hdc);
			Road1F.draw();
			Road1B.draw();
			Road2F.draw();
			Road2B.draw();

			// Drawing panels
			setPanelHDC(hdc);
			Panel1F.draw();
			Panel1B.draw();
			Panel2F.draw();
			Panel2B.draw();

			EndPaint(hWnd, &ps);
			}
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

// Mesage handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}
