/*
 * Project.......: CROSSROAD CONTROL SIMULATOR
 * File..........: traffic_light.cpp
 * Author........: Rogai Davide
 * Last review date: 16/05/2002
 * Description:
 *		Implementation of the traffic light element
 */

#include <windows.h>
#include "traffic_light.h"

extern HWND hWnd;

HDC thdc;

//Retreving the device context
void setTrafficHDC(HDC trafficHdc)
{
	thdc=trafficHdc;
}

//Switching light on/off
void Light::switchLight(bool val)
{
	//it draws only if its state has changed
	if(val!=switch_state)
	{
		switch_state=val;
		thdc = GetDC(hWnd);
		draw();
		ReleaseDC(hWnd,thdc);
	}
}

//Drawing a light
void Light::draw()
{
	HBRUSH oldBrush;

	HBRUSH brush=CreateSolidBrush((switch_state==true)?on_color:off_color);
	oldBrush= (HBRUSH) SelectObject(thdc,brush);
	Ellipse(thdc,x,y,x+LIGHT_SIZE,y+LIGHT_SIZE);
	SelectObject(thdc,oldBrush);
    DeleteObject(brush); 
}

TrafficLight::TrafficLight(int posx, int posy, int trafficdir) 
		: green(GREEN_COLOR), yellow(YELLOW_COLOR), red(RED_COLOR) 
{
	x=posx;
	y=posy;
	dir=trafficdir;
	int movex,movey;
		
	movex=x+LIGHT_BORDER;
	movey=y+LIGHT_BORDER;
	if(!(dir&1))
		green.setPos(movex, movey);
	else
		red.setPos(movex, movey);
	if(!(dir&2))
		movey+=LIGHT_SIZE;
	else
		movex+=LIGHT_SIZE;
	yellow.setPos(movex, movey);
	if(!(dir&2))
		movey+=LIGHT_SIZE;
	else
		movex+=LIGHT_SIZE;
	if(!(dir&1))
		red.setPos(movex, movey);
	else
		green.setPos(movex, movey);
}

//Drawing a complete traffic light
void TrafficLight::draw()
{
	HBRUSH oldBrush;

	HBRUSH blackBrush=CreateSolidBrush(0x00000000);
	oldBrush=(HBRUSH) SelectObject(thdc,blackBrush);
	Rectangle(thdc,x,y,x+2*LIGHT_BORDER+LIGHT_SIZE+((dir&2)?2*LIGHT_SIZE:0),
		y+2*LIGHT_BORDER+LIGHT_SIZE+((dir&2)?0:2*LIGHT_SIZE));
	SelectObject(thdc,oldBrush);
    DeleteObject(blackBrush); 

	green.draw();
	yellow.draw();
	red.draw();
}