/*
 * Project.......: CROSSROAD CONTROL SIMULATOR
 * File..........: traffic_panel.cpp
 * Author........: Rogai Davide
 * Last review date: 16/05/2002
 * Description:
 *		Implementation of the traffic panel (commercial spot, traffic info)
 */

#include <windows.h>
#include "traffic_panel.h"

extern HWND hWnd;

HDC phdc;

//Retreiving the device context
void setPanelHDC(HDC panelHdc)
{
	phdc=panelHdc;
}

TrafficPanel::TrafficPanel(int posx,int posy, int paneldir)
{
	x=posx;
	y=posy;
	dir=paneldir;
	toShow[0]='\0';
}


//Setting the text to show in the panel
void TrafficPanel::setText(const char * text)
{
	strncpy(toShow,text,50);
	toShow[50-1]='\0';
	phdc=GetDC(hWnd);
	draw();
	ReleaseDC(hWnd,phdc);
}

//Drawing the panel at the current state
void TrafficPanel::draw()
{
	HBRUSH oldBrush;

	HBRUSH bkBrush=CreateSolidBrush(PANEL_BK_COLOR);
	oldBrush=(HBRUSH) SelectObject(phdc,bkBrush);
	if(!(dir&2))
		Rectangle(phdc,x,y,x+PANEL_X_SIZE,y+PANEL_Y_SIZE);
	else
		Rectangle(phdc,x,y,x+PANEL_Y_SIZE,y+PANEL_X_SIZE);
	SelectObject(phdc,oldBrush);
    DeleteObject(bkBrush); 

	HFONT hfnt, hfntPrev; 
  
	PLOGFONT plf = (PLOGFONT) LocalAlloc(LPTR, sizeof(LOGFONT)); 

	// Specify a font typeface name and weight. 
 
	lstrcpy(plf->lfFaceName, "Arial"); 
	plf->lfWeight = FW_NORMAL; 
 
	int angle;

	switch(dir)
	{
	case 0:
		angle=0;
		break;
	case 1:
		angle=1800;
		break;
	case 2:
		angle=900;
		break;
	case 3:
		angle=2700;
		break;
	default:
		angle=0;
	}

	plf->lfEscapement = angle;
	hfnt = CreateFontIndirect(plf); 
	hfntPrev = (HFONT) SelectObject(phdc, hfnt); 

	SetBkMode(phdc, TRANSPARENT); 

	int text_x,text_y;

	switch(dir)
	{
	case 0:
	text_x=x+3;
	text_y=y;
	break;
	case 1:
	text_x=x+PANEL_X_SIZE-3;
	text_y=y+PANEL_Y_SIZE;
	break;
	case 2:
	text_x=x;
	text_y=y+PANEL_X_SIZE-3;
	break;
	case 3:
	text_x=x+PANEL_Y_SIZE;
	text_y=y+3;
	break;
	default:
	text_x=x;
	text_y=y;
	}
	TextOut(phdc, text_x, text_y, toShow, 
		(strlen(toShow)<PANEL_CHARS)?strlen(toShow):PANEL_CHARS); 
    SelectObject(phdc, hfntPrev); 
    DeleteObject(hfnt); 

	SetBkMode(phdc, OPAQUE); 

	// Free the memory allocated for the LOGFONT structure. 
 
	LocalFree((LOCALHANDLE) plf); 
}